--Testing for the never forget Model


note
	description : "testSCOOP application root class"
	date        : "$Date$"
	revision    : "$Revision$"

class
	APPLICATION

inherit
	ARGUMENTS

create
	make

feature {NONE} -- Initialization

	make
			-- Run application.
		local
			i, max: INTEGER
			last_percentage, now_percentage: REAL_64
			stop: BOOLEAN
			start_t, end_t: TIME
		do

			create start_t.make_now

--			stop := execute_whole_job(229)

--			stop := True

			from
				i := 0
				max := (12).power(max_calls).floor
				print ("%N%N%NTotal of experiments: " + max.out + "%N")
			until
				i >= max or stop
			loop

				last_percentage := (i-1) / (max / 100)
            	now_percentage := (i) / (max / 100)
            	if last_percentage.floor /= now_percentage.floor then
            		print("%R" + now_percentage.floor.out + "%% done.")
            	end

            	stop := execute_whole_job(i)

            	i := i+1
			end
			print ("%R100%% done.%N")

			create end_t.make_now
			print ("Fine seconds elapsed: " + (end_t.fine_seconds - start_t.fine_seconds).out + "%N")
		end

	execute_whole_job (i: INTEGER): BOOLEAN
		local
			p, q: separate WORKER
			stop: BOOLEAN
			job, job_total, step, step_total: INTEGER
		do
			if stop then
				print ("Whole Job " + i.out + " terminated with an unexpected exception on step " + step.out + ".%N")-- This was executed:%N")
				from
					job_total := i
					step_total := step
					step := 1
				until
					step_total < step
				loop
					job := job_total \\ 12

					if step_total = step then
						print((job+1).out + " (unexpected Exception)%N")
					else
						print((job+1).out + "%N")
					end


					job_total := (job_total - job) // 12
					step := step+1
				end

				Result := True
			else
				print_debug ("Debug: Whole Job " + i.out + "%N")
				create p.make
				create q.make
				p_flag := False
				q_flag := False

				from
					job_total := i
					step := 1
				until
					job_total = 0
				loop
					job := job_total \\ 12

					print_debug ("Debug: Executing job: " + (job+1).out + "%N")
					execute_job(job+1, p, q)	--Acquires and releases locks to P and Q. Since we are in the never-forget system, the exceptions should NOT be dropped!

					job_total := (job_total - job) // 12
					step := step+1
				end

				Result := False
			end
		rescue
			stop := True
			retry
		end

	execute_job (job: INTEGER; p, q: separate WORKER)
		local
			expect_error, do_not_handle, b, stop: BOOLEAN
		do
			if stop then
				--The job can't be accomplished (such as a failing sync call), and has thrown the expected exception.
				--Therefor, move on
			else
				--This is for the system to be implemented
				if job = 4 or job = 6 or job = 10 or job = 12 then
					expect_error := true
				elseif p_flag and (job <= 6 or job = 9) then
					expect_error := true
				elseif q_flag and (job > 6 or job = 3) then
					expect_error := true
				else
					expect_error := false
				end

				inspect
				        job
				when 1 then
					print_debug ("Debug: Executing job 1: Send async call p -> q%N")
					writeln_debug (510)
					b := p.send_async_call (q)
				when 2 then
					if not p_flag then
						print_debug ("Debug: Executing job 2: Send async call p -> q and FAIL. Setting Q dirty.%N")
						writeln_debug (511)
					else
						print_debug ("Debug: Executing job 2: Send async call p -> q and FAIL. Setting Q NOT dirty since P is dirty.%N")
						writeln_debug (530)
					end
					b := p.send_async_call_fail (q)
					--If P is already failed, no call to Q will be logged, and as such Q should not be set to failed-
					if not p_flag then
						q_flag := true
					end
				when 3 then
					print_debug ("Debug: Executing job 3: Send sync call p -> q%N")
					writeln_debug (512)
					b := p.send_sync_call (q)
				when 4 then
					print_debug ("Debug: Executing job 4: Send sync call p -> q and FAIL%N")
					writeln_debug (513)
					b := p.send_sync_call_fail (q)
					--Do not set a flag, sine exception should be immediately returned
				when 5 then
					print_debug ("Debug: Executing job 5: Send self call p -> p%N")
					writeln_debug (514)
					b := p.send_self_call
				when 6 then
					print_debug ("Debug: Executing job 6: Send self call p -> p and FAIL%N")
					writeln_debug (515)
					b := p.send_self_call_fail
					--Do not set a flag, sine exception should be immediately returned
				when 7 then
					print_debug ("Debug: Executing job 7: Send async call q -> p%N")
					writeln_debug (516)
					b := q.send_async_call (p)
				when 8 then
					if not q_flag then
						print_debug ("Debug: Executing job 8: Send async call q -> p and FAIL. Setting P dirty.%N")
						writeln_debug (517)
					else
						print_debug ("Debug: Executing job 8: Send async call q -> p and FAIL. Setting P NOT dirty since Q is dirty.%N")
						writeln_debug (531)
					end
					b := q.send_async_call_fail (p)
					--If Q is already failed, no call to P will be logged, and as such P should not be set to failed-
					if not q_flag then
						p_flag := true
					end
				when 9 then
					print_debug ("Debug: Executing job 9: Send sync call q -> p%N")
					writeln_debug (518)
					b := q.send_sync_call (p)
				when 10 then
					print_debug ("Debug: Executing job 10: Send sync call q -> p and FAIL%N")
					writeln_debug (519)
					b := q.send_sync_call_fail (p)
					--Do not set a flag, sine exception should be immediately returned
				when 11 then
					print_debug ("Debug: Executing job 11: Send self call q -> q%N")
					writeln_debug (520)
					b := q.send_self_call
				when 12 then
					print_debug ("Debug: Executing job 12: Send self call q -> q and FAIL%N")
					writeln_debug (521)
					b := q.send_self_call_fail
					--Do not set a flag, sine exception should be immediately returned
				end

				if expect_error then
					--We have expected an error BUT IT HAS NOT HAPPENED.
					print ("We expected an error BUT IT HAS NOT HAPPENED%N")
					writeln (522)
					do_not_handle := True
					my_exception.raise
				end
				--print ("Result: " + b.out + "%N")

			end
		rescue
			print_debug ("In rescue of APPLICATION.execute_job%N")
			writeln_debug (529)

			if do_not_handle then
				--Well, propagate error
			else
				--This logic is hard to understand...
				if p_flag and job <= 6 then
					p_flag := False
				elseif q_flag and job > 6 then
					q_flag := False
				elseif not p_flag and (job = 3 or job = 4) then
					q_flag := False
				elseif not q_flag and (job = 9 or job = 10) then
					p_flag := False
				end

				if expect_error and (job = 4 or job = 6 or job = 10 or job = 12) then
					stop := True
					retry
				elseif expect_error then
					--We have to reset the Flags now...
					retry
				else
					print ("We DID NOT expected an error BUT IT HAS HAPPENED%N")
					--We have an unexpecter error, do not handle it
					writeln (523)
				end
			end

		end

	max_calls: INTEGER = 4
	p_flag, q_flag: BOOLEAN

	debug_on: BOOLEAN = False

	print_debug (s: STRING)
		do
			if debug_on then
				print(s)
			end
		end

	my_exception: DEVELOPER_EXCEPTION
            once
                create Result
            end

    writeln_debug (s: INTEGER)
    	do
    		if debug_on then
    			writeln (s)
    		end
    	end

    writelnTwoArgs_debug (s, x: INTEGER)
    	do
    		if debug_on then
    			writelnTwoArgs (s, x)
    		end
    	end

	writeln (s: INTEGER)
		external
			"C inline"
		alias
			"writeln($s);"
		end

	writelnTwoArgs (s, x: INTEGER)
		external
			"C inline"
		alias
			"writelnTwoArgs($s, $x);"
		end
end
