--Testing for Safemode


note
	description : "testSCOOP application root class"
	date        : "$Date$"
	revision    : "$Revision$"

class
	APPLICATION

inherit
	ARGUMENTS

create
	make

feature {NONE} -- Initialization

	make
			-- Run application.
		local
			i, max: INTEGER
			last_percentage, now_percentage: REAL_64
			stop: BOOLEAN
			start_t, end_t: TIME
		do

			create start_t.make_now
--			stop := execute_whole_job(0)

--			stop := True

			from
				i := 0
				max := (12).power(max_calls).floor
				print ("%N%N%NTotal of experiments: " + max.out + "%N")
			until
				i >= max or stop
			loop

				last_percentage := (i-1) / (max / 100)
            	now_percentage := (i) / (max / 100)
            	if last_percentage.floor /= now_percentage.floor then
            		print("%R" + now_percentage.floor.out + "%% done.")
            	end

            	stop := execute_whole_job(i)

            	i := i+1
			end
			print ("%R100%% done.%N")

			create end_t.make_now
			print ("Fine seconds elapsed: " + (end_t.fine_seconds - start_t.fine_seconds).out + "%N")
		end

	execute_whole_job (i: INTEGER): BOOLEAN
		local
			p, q: separate WORKER
			stop: BOOLEAN
		do
			if not stop then

				print_debug ("Debug: Whole Job " + i.out + "%N")
				create p.make
				create q.make
				p_flag := False
				q_flag := False

				Result := execute_whole_job_internal(p, q, i)

			end
		rescue
			print ("Reached Max. SCOOP Processors. Last job was: " + i.out + "%N")
			stop := True
			Result := True
			retry
		end

	--This is used so the locks on P and Q are never released!
	--Safemode works on Client's FEATURE BODY END (and NOT on LOCK RELEASE)
	execute_whole_job_internal(p, q: separate WORKER; i: INTEGER): BOOLEAN
		local
			stop, res: BOOLEAN
			job, job_total, step, step_total: INTEGER
		do
			if stop then
				print ("Job " + i.out + " terminated with an unexpected exception on step " + step.out + ".%N")-- This was executed:%N")
				from
					job_total := i
					step_total := step
					step := 1
				until
					step_total < step
				loop
					job := job_total \\ 12

					if step_total = step then
						print((job+1).out + " (unexpected Exception)%N")
					else
						print((job+1).out + "%N")
					end


					job_total := (job_total - job) // 12
					step := step+1
				end

				Result := True
			else
				from
					job_total := i
					step := 1
				until
					job_total = 0
				loop
					job := job_total \\ 12

					print_debug ("Debug: Executing job: " + (job+1).out + "%N")
					res := execute_job(job+1, p, q)
					if not res and (job+1 = 2 or job+1 = 4 or job+1 = 6 or job+1 = 8 or job+1 = 10 or job+1 = 12) then
						print ("We expected an error BUT IT HAS NOT HAPPENED%N")
						writeln (522)
						my_exception.raise
					end

					job_total := (job_total - job) // 12
					step := step+1
				end

				Result := False
			end
		rescue
			stop := True
			retry
		end

	--Hm, this setup is bad - every call will immediately lead to a check after it...
	--Also: What if we want a call to fail, but it doesn't? -> The actual check happens in execute_whole_job_internal after the call to this feature
	execute_job (job: INTEGER; p, q: separate WORKER): BOOLEAN
		local
			has_been_in_rescue, b, stop: BOOLEAN
		do
			if not stop then
				--The job can't be accomplished (such as a failing sync call), and has thrown the expected exception.

				inspect
				        job
				when 1 then
					print_debug ("Debug: Executing job 1: Send async call p -> q%N")
					writeln (510)
					b := p.send_async_call (q)
				when 2 then
					print_debug ("Debug: Executing job 2: Send async call p -> q and FAIL. Setting Q dirty.%N")
					writeln (511)
					b := p.send_async_call_fail (q)
					--Do not set a flag, sine exception should be immediately returned
				when 3 then
					print_debug ("Debug: Executing job 3: Send sync call p -> q%N")
					writeln (512)
					b := p.send_sync_call (q)
				when 4 then
					print_debug ("Debug: Executing job 4: Send sync call p -> q and FAIL%N")
					writeln (513)
					b := p.send_sync_call_fail (q)
					--Do not set a flag, sine exception should be immediately returned
				when 5 then
					print_debug ("Debug: Executing job 5: Send self call p -> p%N")
					writeln (514)
					b := p.send_self_call
				when 6 then
					print_debug ("Debug: Executing job 6: Send self call p -> p and FAIL%N")
					writeln (515)
					b := p.send_self_call_fail
					--Do not set a flag, sine exception should be immediately returned
				when 7 then
					print_debug ("Debug: Executing job 7: Send async call q -> p%N")
					writeln (516)
					b := q.send_async_call (p)
				when 8 then
					print_debug ("Debug: Executing job 8: Send async call q -> p and FAIL. Setting P dirty.%N")
					writeln (517)
					b := q.send_async_call_fail (p)
					--Do not set a flag, sine exception should be immediately returned
				when 9 then
					print_debug ("Debug: Executing job 9: Send sync call q -> p%N")
					writeln (518)
					b := q.send_sync_call (p)
				when 10 then
					print_debug ("Debug: Executing job 10: Send sync call q -> p and FAIL%N")
					writeln (519)
					b := q.send_sync_call_fail (p)
					--Do not set a flag, sine exception should be immediately returned
				when 11 then
					print_debug ("Debug: Executing job 11: Send self call q -> q%N")
					writeln (520)
					b := q.send_self_call
				when 12 then
					print_debug ("Debug: Executing job 12: Send self call q -> q and FAIL%N")
					writeln (521)
					b := q.send_self_call_fail
					--Do not set a flag, sine exception should be immediately returned
				end

			end
			Result := has_been_in_rescue
		rescue
			print_debug ("In rescue of APPLICATION.execute_job%N")
			writeln (529)

			has_been_in_rescue := True

			if job = 2 or job = 4 or job = 6 or job = 8 or job = 10 or job = 12 then
				stop := True
				retry
			else
				print ("We DID NOT expected an error BUT IT HAS HAPPENED%N")
				--We have an unexpecter error, do not handle it
				writeln (523)
			end

		end

	max_calls: INTEGER = 4
	sleep_time: INTEGER_64 = 100_000_000
	p_flag, q_flag: BOOLEAN

	debug_on: BOOLEAN = True

	print_debug (s: STRING)
		do
			if debug_on then
				print(s)
			end
		end

	my_exception: DEVELOPER_EXCEPTION
            once
                create Result
            end
	writeln (s: INTEGER)
		external
			"C inline"
		alias
			"writeln($s);"
		end

	writelnTwoArgs (s, x: INTEGER)
		external
			"C inline"
		alias
			"writelnTwoArgs($s, $x);"
		end
end
