note
	description	: "Objects that implement producers."
	author		: "Volkan Arslan, Yann Mueller, Piotr Nienaltowski."
	date		: "$Date: 18.05.2007$"
	revision	: "1.0.0"

class
	PRODUCER

create
	make_with_buffer

feature -- Initialization

	make_with_buffer (a_buffer: separate BOUNDED_BUFFER [INTEGER]; a_id: INTEGER)
			-- Initialize with identifier `a_id' to produce into `a_buffer'.
		require
			valid_id: a_id > 0
		do
			buffer := a_buffer
			id := a_id
		ensure
			id_set: id = a_id
		end

feature -- Basic operations

	produce (n: INTEGER)
			-- Produce n elements.
		require
			valid_n: n >= 0
		local
			l_element: INTEGER
			i: INTEGER
		do
			from

			until
				i >= n
			loop
				l_element := id * 1000000 + i
				--print ("%NPRODUCER " + id.out + ": Attempting to store element " + l_element.out + " ...")
				store (buffer, l_element)
				print ("%NPRODUCER " + id.out + ": I've just stored element " + l_element.out + ". I have now produced " + (i+1).out + " elements out of " + n.out + ".")

				i := i + 1
			end
		rescue
			--print ("%NPRODUCER " + id.out + ": In Rescue of produce...")
			retry
		end

feature {NONE}

	store (a_buffer: separate BOUNDED_BUFFER [INTEGER]; an_element: INTEGER)
			-- Store `an_element' into `a_buffer'.
--		require
--			not a_buffer.is_full
		local
			b: BOOLEAN
		do
			a_buffer.put (an_element)
--		ensure
--			not a_buffer.is_empty
--			a_buffer.count = old a_buffer.count + 1
		rescue
			--Aw, we couldn't put
			--print ("%NPRODUCER " + id.out + ": FAILED to store element " + an_element.out + " ...")
		end

feature {NONE} -- Implementation

	buffer: separate BOUNDED_BUFFER [INTEGER]
			-- Shared product buffer.

	id: INTEGER
			-- Unique identifier.
invariant
	valid_id: id > 0

end -- class PRODUCER

